#include "OpenGLWndEngine.h"

COpenGLWndEngine::COpenGLWndEngine()
{
	nbullets=0;
	TemplePos[0]=0;
	TemplePos[1]=0;
	TemplePos[2]=0;

	SpaceShip[0]=0;
	SpaceShip[1]=0;
	SpaceShip[2]=0;

	DrawTemple=FALSE;
	
	mass=10;


}

COpenGLWndEngine::~COpenGLWndEngine()
{

}

/******************************************************************************************/
/*                                                                                        */
/* OPENGL Initialization Functions for Windows                                            */
/*                                                                                        */
/******************************************************************************************/

void COpenGLWndEngine::EnableOpenGL(HWND hWnd, HDC * hDC, HGLRC * hRC)
{
    int nPixelFormat;
    static PIXELFORMATDESCRIPTOR pfd = {
        sizeof (PIXELFORMATDESCRIPTOR),             // Size of this structure
        1,                                          // Version number
        PFD_DRAW_TO_WINDOW |                        // Flags
        PFD_SUPPORT_OPENGL |
        PFD_GENERIC_ACCELERATED|
        PFD_DOUBLEBUFFER,
        PFD_TYPE_RGBA,                              // RGBA pixel values
        24,                                         // 24-bit color
        0, 0, 0, 0, 0, 0,                           // Don't care about these
        0, 0,                                       // No alpha buffer
        0, 0, 0, 0, 0,                              // No accumulation buffer
        32,                                         // 32-bit depth buffer
        0,                                          // No stencil buffer
        0,                                          // No auxiliary buffers
        PFD_MAIN_PLANE,                             // Layer type
        0,                                          // Reserved (must be 0)
        0, 0, 0                                     // No layer masks
    };
	
	*hDC = GetDC( hWnd );
	dc=*hDC;
    nPixelFormat = ChoosePixelFormat (*hDC, &pfd);
    SetPixelFormat (*hDC, nPixelFormat, &pfd);

    *hRC = wglCreateContext( *hDC );
    wglMakeCurrent( *hDC, *hRC );
}

/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::DisableOpenGL(HWND hWnd, HDC hDC, HGLRC hRC)
{
    wglMakeCurrent( NULL, NULL );
    wglDeleteContext( hRC );
    ReleaseDC( hWnd, hDC );
}

/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::ClearWindow(float r,float g,float b,float lum,float depth)
{
	glClearColor (r, g, b, lum);		// Sets the Clear color of the back Surface
	glClearDepth(depth);				// Sets the Depth Buffer Values
	glClear (GL_COLOR_BUFFER_BIT| GL_DEPTH_BUFFER_BIT);
	
}
/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::WindowResized(int w,int h)
{
	glViewport (0,0,(GLsizei) w, (GLsizei) h);	// Define the size of the 2D rectangle to have the scene
	this->w=w;
	this->h=h;
	
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity();
	glFrustum (-5.0,5.0,-5.0,5.0,5,700.0);	// Define the projection of the lens
	glMatrixMode(GL_MODELVIEW);
	
	glClear (GL_COLOR_BUFFER_BIT| GL_DEPTH_BUFFER_BIT);

	
}

/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::Flip()
{
	SwapBuffers(dc);
}

/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::Switch3D2D(int dir)
{
	if (dir==0) //From 3d we mswitch to 2D
	{
		 glMatrixMode(GL_PROJECTION);
		 glPushMatrix();
		 glLoadIdentity();
		 gluOrtho2D(0, w, h,0);
		 glMatrixMode(GL_MODELVIEW);
		 glPushMatrix();
		 glLoadIdentity();

	}
	if (dir==1) // From 2d again to 3D
	{
		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glPopMatrix();
	}
}
/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::SetSceneGlobals()
{
	GLfloat fogcolor[3]={0.5,0.5,0.5};
	
	glEnable(GL_DEPTH_TEST);

	glFogfv(GL_FOG_COLOR,fogcolor);
	glFogf(GL_FOG_START,30.0);
	glFogf(GL_FOG_END,100.0);
	glFogi(GL_FOG_MODE,GL_LINEAR);

	//Photo.data=LoadTGA("test.tga",&Photo.width,&Photo.height,&Photo.components,&Photo.format);
	//Terrain.data=LoadTGA("Terrain.tga",&Terrain.width,&Terrain.height,&Terrain.components,&Terrain.format);
	//if (Photo.data==NULL) exit(0);
	//LoadTexture();

	hlist1=Gen3DObjectList1();
	//CreateTerrain();
	CreateFonts();

	//	glEnable (GL_CULL_FACE);
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE ,1);
}

/******************************************************************************************/
/*                                                                                        */
/* OPENGL Loading and Creating Materials/Textures                                         */
/*                                                                                        */
/******************************************************************************************/

void COpenGLWndEngine::CreateFonts()
{
	HFONT font;
	LOGFONT logfont;

	logfont.lfHeight=-20;
	logfont.lfWidth=0;
	logfont.lfEscapement=0;
	logfont.lfOrientation=0;
	logfont.lfWeight=FW_BOLD;
	logfont.lfItalic=FALSE;
	logfont.lfUnderline=FALSE;
	logfont.lfStrikeOut=FALSE;
	logfont.lfCharSet=ANSI_CHARSET;
	logfont.lfOutPrecision=OUT_DEFAULT_PRECIS;
	logfont.lfClipPrecision=CLIP_DEFAULT_PRECIS;
	logfont.lfQuality=DEFAULT_QUALITY;
	logfont.lfPitchAndFamily=DEFAULT_PITCH;
	strcpy(logfont.lfFaceName,"Arial");

	font=CreateFontIndirect(&logfont);
	SelectObject(dc,font);

	fontlist=glGenLists(128);
	wglUseFontBitmaps(dc,0,128,fontlist);

	DeleteObject(font);
}

void COpenGLWndEngine::SelectMaterial(MATERIAL *materials,int i)
{
  //glEnd();
  GLfloat alpha=materials[i].alpha;
  MyMaterial (GL_AMBIENT, materials[i].ambient,alpha);
  MyMaterial (GL_DIFFUSE, materials[i].diffuse,alpha);
  MyMaterial (GL_SPECULAR, materials[i].specular,alpha);
  MyMaterial (GL_EMISSION, materials[i].emission,alpha);
  glMaterialf (GL_FRONT_AND_BACK,GL_SHININESS,materials[i].phExp);
  //glBegin(GL_TRIANGLES);

};

/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::MyMaterial(GLenum mode,GLfloat *f,GLfloat alpha)
{
 GLfloat d[4];
 d[0]=f[0];
 d[1]=f[1];
 d[2]=f[2];
 d[3]=alpha;
 glMaterialfv (GL_FRONT_AND_BACK,mode,d);
}

/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::LoadTexture()
{
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,Photo.width,Photo.height,0,Photo.format,GL_UNSIGNED_BYTE,Photo.data);

	glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);

}

/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::CreateTerrain()
{
	GLfloat x,y;
	GLfloat h;
	GLbyte temp;

	h=0;
	for (x=0;x<TERRAIN_SIZE;x++)
		for (y=0;y<TERRAIN_SIZE;y++)
		{
			temp=Terrain.data[(int)x*3*TERRAIN_SIZE+(int)y*3];
			h=temp;
			heights[(int)x][(int)y]=h;
		}

    TerrainList=glGenLists(1);

	glNewList(TerrainList,GL_COMPILE);
	DrawTerrain();
	glEndList();
}

/******************************************************************************************/
/******************************************************************************************/

GLbyte * COpenGLWndEngine::LoadTGA(const char *szFileName, GLint *iWidth, GLint *iHeight, GLint *iComponents, GLenum *eFormat)
{
	int i;
	i=sizeof(TGAHEADER);
    FILE *pFile;				// File pointer
    TGAHEADER tgaHeader;		// TGA file header
    unsigned long lImageSize;	// Size in bytes of image
    short sDepth;				// Pixel depth;
    GLbyte	*pBits = NULL;      // Pointer to bits
    
    // Default/Failed values
    *iWidth = 0;
    *iHeight = 0;
    *eFormat = GL_BGR_EXT;
    *iComponents = GL_RGB8;
    
    // Attempt to open the fil
    pFile = fopen(szFileName, "rb");
    if(pFile == NULL)
        return NULL;
        
    // Read in header (binary)
    fread(&tgaHeader.identsize, 1, 1, pFile);
	fread(&tgaHeader.colorMapBits, 1, 1, pFile);
	fread(&tgaHeader.imageType, 1, 1, pFile);
	fread(&tgaHeader.colorMapStart, 2, 1, pFile);
	fread(&tgaHeader.colorMapLength, 2, 1, pFile);
	fread(&tgaHeader.colorMapType, 1, 1, pFile);
	fread(&tgaHeader.xstart, 2, 1, pFile);
	fread(&tgaHeader.ystart, 2, 1, pFile);
	fread(&tgaHeader.width, 2, 1, pFile);
	fread(&tgaHeader.height, 2, 1, pFile);
	fread(&tgaHeader.bits, 1, 1, pFile);
	fread(&tgaHeader.descriptor, 1, 1, pFile);

    
    // Do byte swap for big vs little endian
#ifdef __APPLE__
    BYTE_SWAP(tgaHeader.colorMapStart);
    BYTE_SWAP(tgaHeader.colorMapLength);
    BYTE_SWAP(tgaHeader.xstart);
    BYTE_SWAP(tgaHeader.ystart);
    BYTE_SWAP(tgaHeader.width);
    BYTE_SWAP(tgaHeader.height);
#endif

        
    // Get width, height, and depth of texture
    *iWidth = tgaHeader.width;
    *iHeight = tgaHeader.height;
    sDepth = tgaHeader.bits / 8;
    
    // Put some validity checks here. Very simply, I only understand
    // or care about 8, 24, or 32 bit targa's.
    if(tgaHeader.bits != 8 && tgaHeader.bits != 24 && tgaHeader.bits != 32)
        return NULL;

    // Calculate size of image buffer
    lImageSize = tgaHeader.width * tgaHeader.height * sDepth;
    
    // Allocate memory and check for success
    pBits = (GLbyte *)malloc(lImageSize * sizeof(GLbyte));
    if(pBits == NULL)
        return NULL;
    
    // Read in the bits
    // Check for read error. This should catch RLE or other 
    // weird formats that I don't want to recognize
    if(fread(pBits, lImageSize, 1, pFile) != 1)
        {
        free(pBits);
        return NULL;
        }
    
    // Set OpenGL format expected
    switch(sDepth)
        {
        case 3:     // Most likely case
            *eFormat = GL_RGB;
            *iComponents = GL_RGB8;
            break;
        case 4:
            *eFormat = GL_BGRA_EXT;
            *iComponents = GL_RGBA8;
            break;
        case 1:
            *eFormat = GL_LUMINANCE;
            *iComponents = GL_LUMINANCE8;
            break;
        };
        
    
    // Done with File
    fclose(pFile);
        
    // Return pointer to image data
    return pBits;
}

/******************************************************************************************/
/*                                                                                        */
/* OPENGL Camera Manipuation                                                              */
/*                                                                                        */
/******************************************************************************************/

void COpenGLWndEngine::SetCamera (GLfloat xpos,GLfloat ypos,GLfloat zpos)
{
	
	Player.Pos[0]=xpos;Player.Pos[1]=ypos;Player.Pos[2]=zpos;
	Player.Front[0]=0;Player.Front[1]=0;Player.Front[2]=1;
	Player.Up[0]=0;Player.Up[1]=1;Player.Up[2]=0;
	Player.Right[0]=1;Player.Right[1]=0;Player.Right[2]=0;
	Player.Theta[0]=0;Player.Theta[1]=0;Player.Theta[2]=0;

	glLoadIdentity();
	gluLookAt(Player.Pos[0],Player.Pos[1],Player.Pos[2],
		      Player.Pos[0]-Player.Front[0],Player.Pos[1]-Player.Front[1],Player.Pos[2]-Player.Front[2],
			  0,1,0);

	
}


/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::MoveCamera (int dir,GLfloat step)
{
	if (dir==0)  // LEFT
	{
		Player.Pos[0]-=(step)*Player.Right[0]; Player.Pos[1]-=(step)*Player.Right[1]; Player.Pos[2]-=(step)*Player.Right[2];

	}
	if (dir==1)	 // RIGHT
	{
		Player.Pos[0]+=(step)*Player.Right[0]; Player.Pos[1]+=(step)*Player.Right[1]; Player.Pos[2]+=(step)*Player.Right[2];
	}
	if (dir==2)	 // FORWARD
	{
		Player.Pos[0]-=(step)*Player.Front[0]; Player.Pos[1]-=(step)*Player.Front[1]; Player.Pos[2]-=(step)*Player.Front[2];

	}
	if (dir==3)  // BACKWARD
	{
		Player.Pos[0]+=(step)*Player.Front[0]; Player.Pos[1]+=(step)*Player.Front[1]; Player.Pos[2]+=(step)*Player.Front[2];
	}
	
	if (Player.Pos[0]<5) Player.Pos[0]=5;
	if (Player.Pos[0]>world_size-5) Player.Pos[0]=world_size-5;

	if (Player.Pos[1]<5) Player.Pos[1]=5;
	if (Player.Pos[1]>world_size-5) Player.Pos[1]=world_size-5;

	if (Player.Pos[2]<5) Player.Pos[2]=5;
	if (Player.Pos[2]>world_size-5) Player.Pos[2]=world_size-5;

	glLoadIdentity();
	gluLookAt(Player.Pos[0],Player.Pos[1],Player.Pos[2],
		      Player.Pos[0]-Player.Front[0],Player.Pos[1]-Player.Front[1],Player.Pos[2]-Player.Front[2],
			  0,1,0);


}

/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::RollCamera (int dir, GLfloat step)
{
	if (dir==3) Player.Theta[0]+=step;
	if (dir==2) Player.Theta[0]-=step;

	if (dir==1) Player.Theta[1]-=step;
	if (dir==0) Player.Theta[1]+=step;

	if (Player.Theta[0]>=90) Player.Theta[0]=89;
	if (Player.Theta[0]<=-90) Player.Theta[0]=-89;
	
	Player.Front[0]=cos(Player.Theta[0]*PI/180.0)*sin(Player.Theta[1]*PI/180.0);
	Player.Front[1]=sin(Player.Theta[0]*PI/180.0);
	Player.Front[2]=cos(Player.Theta[0]*PI/180.0)*cos(Player.Theta[1]*PI/180.0);

	Player.Right[0]=cos(Player.Theta[1]*PI/180.0);
	Player.Right[1]=0;
	Player.Right[2]=-sin(Player.Theta[1]*PI/180.0);

	Player.Up[0]=0;
	Player.Up[1]=cos(Player.Theta[1]*PI/180.0);
	Player.Up[2]=-sin(Player.Theta[0]*PI/180.0);

	glLoadIdentity();
	gluLookAt(Player.Pos[0],Player.Pos[1],Player.Pos[2],
		      Player.Pos[0]-Player.Front[0],Player.Pos[1]-Player.Front[1],Player.Pos[2]-Player.Front[2],
			  0,1,0);

}


/******************************************************************************************/
/*                                                                                        */
/* OPENGL Lights Creation                                                                */
/*                                                                                        */
/******************************************************************************************/

void COpenGLWndEngine::SetLights()
{

	// Create the SCENE'S AMBIENT LIGHT
	General.ambient[0]=0.2;General.ambient[1]=0.2;General.ambient[2]=0.2;General.ambient[3]=1;

	General.diffuse[0]=0;General.diffuse[1]=0;General.diffuse[2]=0;General.diffuse[3]=1;

	General.specular[0]=0;General.specular[1]=0;General.specular[2]=0;General.specular[3]=1;

	General.pos[0]=250;General.pos[1]=250;General.pos[2]=500;General.pos[3]=1;

	glLightfv(GL_LIGHT0,GL_AMBIENT,General.ambient);
	glLightfv(GL_LIGHT0,GL_SPECULAR,General.specular);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,General.diffuse);

	glEnable(GL_LIGHT0);


	// Create the SCENE'S SPOT LIGHT
	Spot.ambient[0]=0.2;Spot.ambient[1]=0.2;Spot.ambient[2]=0.2;Spot.ambient[3]=1;

	Spot.specular[0]=0.5;Spot.specular[1]=0.5;Spot.specular[2]=0.5;Spot.specular[3]=1;

	Spot.diffuse[0]=0.5;Spot.diffuse[1]=0.5;Spot.diffuse[2]=0.5;Spot.diffuse[3]=0.5;

	Spot.pos[0]=0;Spot.pos[1]=0;Spot.pos[2]=0;Spot.pos[3]=1;


	glLightfv(GL_LIGHT1,GL_AMBIENT,Spot.ambient);
	glLightfv(GL_LIGHT1,GL_SPECULAR,Spot.specular);
	glLightfv(GL_LIGHT1,GL_DIFFUSE,Spot.diffuse);
	glLightf(GL_LIGHT1,GL_SPOT_CUTOFF,60);

	glEnable(GL_LIGHT1);

	// Create the SCENE'S SPOT LIGHT
	SpotFixed.ambient[0]=0.2;SpotFixed.ambient[1]=0.2;SpotFixed.ambient[2]=0.2;SpotFixed.ambient[3]=1;

	SpotFixed.specular[0]=1.2;SpotFixed.specular[1]=1.2;SpotFixed.specular[2]=1.2;SpotFixed.specular[3]=1;

	SpotFixed.diffuse[0]=0.9;SpotFixed.diffuse[1]=0.9;SpotFixed.diffuse[2]=0.9;SpotFixed.diffuse[3]=1;

	SpotFixed.pos[0]=0;SpotFixed.pos[1]=0;SpotFixed.pos[2]=0;SpotFixed.pos[3]=1;


	glLightfv(GL_LIGHT2,GL_AMBIENT,SpotFixed.ambient);
	glLightfv(GL_LIGHT2,GL_SPECULAR,SpotFixed.specular);
	glLightfv(GL_LIGHT2,GL_DIFFUSE,SpotFixed.diffuse);
	glLightf(GL_LIGHT2,GL_SPOT_CUTOFF,80);

	glEnable(GL_LIGHT2);

	//Let there be light !

	glEnable(GL_LIGHTING);
}

/******************************************************************************************/
/*                                                                                        */
/* OPENGL Application Related Objects                                                     */
/*                                                                                        */
/******************************************************************************************/

void COpenGLWndEngine::DrawObjects ()
{

	/*********************************************/
	/* SAVE THE VIEWING MATRIX                   */
	/*********************************************/
	glPushMatrix();	
	/*********************************************/
	/* SET THE SPOT LIGHT OVER OUR HEAD          */
	/*********************************************/

	glLightfv(GL_LIGHT0,GL_POSITION,General.pos);
	SpotFixed.pos[0]=250;
	SpotFixed.pos[1]=100;
	SpotFixed.pos[2]=490;
	glLightfv(GL_LIGHT2,GL_POSITION,SpotFixed.pos);
	SpotFixed.dir[0]=0;
	SpotFixed.dir[1]=0;
	SpotFixed.dir[2]=-1;
	glLightfv(GL_LIGHT2,GL_SPOT_DIRECTION,SpotFixed.dir);
	
	
	glLoadIdentity();
	Spot.pos[0]=0;
	Spot.pos[1]=0;
	Spot.pos[2]=0;
	glLightfv(GL_LIGHT1,GL_POSITION,Spot.pos);
	Spot.dir[0]=0;
	Spot.dir[1]=0;
	Spot.dir[2]=-1;
	glLightfv(GL_LIGHT1,GL_SPOT_DIRECTION,Spot.dir);

	/*********************************************/
	/* RESTORE THE VIEWING MAT                   */
	/*********************************************/
	glPopMatrix();
	/*********************************************/
	/* SAVE THE VIEWING MATRIX                   */
	/*********************************************/
	glPushMatrix();
		
	    //DrawTerrain();
		//glCallList(TerrainList);
		DrawCube();
		DrawBullets();
		if (DrawTemple) DrawEggHead();
		DrawSphere();
		
		char buf[100];
		Switch3D2D(0);

			glDisable(GL_LIGHTING);
			glColor3f(1.0,1.0,1.0);
			glListBase(fontlist);
			SelectMaterial(Appmaterials,4);
			glRasterPos2i(20,20);
			sprintf(buf,"Pos: %.0f,%.0f,%.0f",Player.Pos[0],Player.Pos[1],Player.Pos[2]);
			glCallLists(strlen(buf),GL_UNSIGNED_BYTE,buf);
			glBegin(GL_LINES);
			glVertex2f(0,0);
			glVertex2f(w,h);
			glVertex2f(w,0);
			glVertex2f(0,h);
		
			glEnd();
		Switch3D2D(1);
		glEnable(GL_LIGHTING);
		DrawWalls(500);

	glPopMatrix();
	
	/*********************************************/
	/* UNCOMMENT FOR FOG                         */
	/*********************************************/

	// if (Player.Pos[2]>100) glEnable(GL_FOG);
	// else glDisable(GL_FOG);

  }


/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::CreateBullet()
{
	if (nbullets<MAX_BULLETS)
	{
		Bullets[nbullets]=Player;
		BulletTimer[nbullets]=BULLET_LIFE;
		nbullets++;
	}
	else
	{
		nbullets=0;
	}
}

void COpenGLWndEngine::UpdateBullets()
{
	int i;
	for (i=0;i<nbullets;i++)
	{
		if (BulletTimer[i]>0)
		{
			Bullets[i].Pos[0]-=7*Bullets[i].Front[0];
			Bullets[i].Pos[1]-=7*Bullets[i].Front[1];
			Bullets[i].Pos[2]-=7*Bullets[i].Front[2];

			if ((Bullets[i].Pos[1]<=2)||(Bullets[i].Pos[1]>=world_size-2))
				Bullets[i].Front[1]=-Bullets[i].Front[1];


			if ((Bullets[i].Pos[2]<=2)||(Bullets[i].Pos[2]>=world_size-2))
				Bullets[i].Front[2]=-Bullets[i].Front[2];

			if ((Bullets[i].Pos[0]<=2)||(Bullets[i].Pos[0]>=world_size-2))
				Bullets[i].Front[0]=-Bullets[i].Front[0];


			BulletTimer[i]--;
		}
	}
}

void COpenGLWndEngine::DrawBullets()
{
	int i;

	SelectMaterial(Appmaterials,5);
	

	for (i=0;i<nbullets;i++)
	{
		if (BulletTimer[i]>0)
		{
			glPushMatrix();
			glTranslatef(Bullets[i].Pos[0],Bullets[i].Pos[1],Bullets[i].Pos[2]);
			glutSolidSphere (1,10,10);
			glPopMatrix();
		}
	}
	

}

/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::DrawCube()
{


	SelectMaterial(Appmaterials,1);

	glPushMatrix();
	
	glTranslatef(TemplePos[0],TemplePos[1],TemplePos[2]);
	glutSolidCube(12);
	
	glPopMatrix();

}

/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::DrawSphere()
{
	

	//glBlendFunc(GL_ONE_MINUS_SRC_ALPHA,GL_SRC_ALPHA);
	//glEnable(GL_BLEND);

	SelectMaterial(Appmaterials,2);

	glPushMatrix();

		glTranslatef(SpaceShip[0],SpaceShip[1],SpaceShip[2]);
		glutSolidSphere (mass,100,100);
	
	glPopMatrix();

	//glDisable(GL_BLEND);

}

/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::DrawEggHead()
{
	//glEnable(GL_TEXTURE_2D);	
	glPushMatrix();

		
		
		glTranslatef(TemplePos[0],TemplePos[1],TemplePos[2]);
		glRotatef(-90,1,0,0);
		glCallList(hlist1);
		
	//	glTranslatef(0,0,100);
	//	glCallList(hlist);
		
	glPopMatrix();

	//glDisable(GL_TEXTURE_2D);
}

/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::DrawWalls (GLfloat size)
{

	glBlendFunc(GL_ONE_MINUS_SRC_ALPHA,GL_SRC_ALPHA);
	glEnable(GL_BLEND);

	SelectMaterial(Appmaterials,3);

	world_size=size;
	
	//glEnable(GL_TEXTURE_2D);
	glShadeModel(GL_SMOOTH);
	glBegin (GL_POLYGON);
		glNormal3f(1,0,0);
	    glColor3f(0,0,0);
		glTexCoord2f(0,0);
		glVertex3f(0,0,0);
		
		glColor3f(0,0,0);
		glTexCoord2f(1,0);
		glVertex3f(0,size,0);
		
		glColor3f(0,0,1);
		glTexCoord2f(1,1);
		glVertex3f(0,size,size);
		
		glColor3f(0,0,1);
		glTexCoord2f(0,1);
		glVertex3f(0,0,size);
	glEnd();

	glBegin (GL_POLYGON);
		glNormal3f(0,1,0);

		glColor3f(0,0,0);
		glTexCoord2f(0,0);
		glVertex3f(0,0,0);
		
		glColor3f(0,0,1);
		glTexCoord2f(0,1);
		glVertex3f(size,0,0);
		
		glColor3f(0,0,1);
		glTexCoord2f(1,0);
		glVertex3f(size,size,0);
		
		glColor3f(0,0,0);
		glTexCoord2f(1,1);
		glVertex3f(0,size,0);
	glEnd();

	glBegin (GL_POLYGON);
		glColor3f(0,1,0);
		glTexCoord2f(0,0);
		glVertex3f(0,0,0);
		glColor3f(0,1,0);
		glTexCoord2f(0,1);
		glVertex3f(0,0,size);
		glColor3f(0,1,0);
		glTexCoord2f(1,1);
		glVertex3f(size,0,size);
		glColor3f(0,1,0);
		glTexCoord2f(1,0);
		glVertex3f(size,0,0);
	glEnd();

/*	glBegin (GL_POLYGON);
		glColor3f(0,1,0);
		glTexCoord2f(0,0);
		glVertex3f(0,0,size);
		glColor3f(0,1,0);
		glTexCoord2f(0,1);
		glVertex3f(size,0,size);
		glColor3f(0,1,0);
		glTexCoord2f(1,1);
		glVertex3f(size,size,size);
		glColor3f(0,1,0);
		glTexCoord2f(1,0);
		glVertex3f(0,size,size);
	glEnd();

	glBegin (GL_POLYGON);
		glColor3f(0,1,0);
		glTexCoord2f(0,0);
		glVertex3f(size,size,0);
		glColor3f(0,1,0);
		glTexCoord2f(0,1);
		glVertex3f(size,size,size);
		glColor3f(0,1,0);
		glTexCoord2f(1,1);
		glVertex3f(size,0,size);
		glColor3f(0,1,0);
		glTexCoord2f(1,0);
		glVertex3f(size,0,0);
	glEnd();*/

//	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
}

/******************************************************************************************/
/******************************************************************************************/

void COpenGLWndEngine::DrawTerrain()
{
	GLfloat Ambient [4]={0,0.3,0,1};
	GLfloat Specular [4]={0,0,0,1};
	GLfloat Diffuse [4]={0,0.4,0,1};

	glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,Ambient);
	glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,Diffuse);
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,Specular);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,130);

	int x,z;

	SCALE=500/TERRAIN_SIZE;

	glBegin(GL_QUADS);
	for (x=0;x<TERRAIN_SIZE-1;x++)
		for (z=0;z<TERRAIN_SIZE-1;z++)
		{
			glNormal3f(0,1,0);
			glVertex3f(SCALE*(x+1),SCALE*heights[x+1][z],SCALE*z);
			glVertex3f(SCALE*(x),SCALE*heights[x][z],SCALE*z);
			glVertex3f(SCALE*(x),SCALE*heights[x][z+1],SCALE*(z+1));
			glVertex3f(SCALE*(x+1),SCALE*heights[x+1][z+1],SCALE*(z+1));

		}

	glEnd();


}


GLint COpenGLWndEngine::Gen3DObjectList1()
{
 int i;
 int j;
 int mcount=0;
 int mindex=0;

 GLint lid=glGenLists(1);  //Get a new list ID

 glNewList(lid, GL_COMPILE); //Create the list

 glBegin (GL_TRIANGLES);
   for(i=0;i<sizeof(Temple_face_indicies)/sizeof(Temple_face_indicies[0]);i++)
   {
      if(!mcount)
       {
        SelectMaterial(Temple_materials,Temple_material_ref[mindex][0]);
        mcount=Temple_material_ref[mindex][1];
        mindex++;
       }
       mcount--;
       for(j=0;j<3;j++)
        {
          int vi=Temple_face_indicies[i][j];
          int ni=Temple_face_indicies[i][j+3];//Normal index
          int ti=Temple_face_indicies[i][j+6];//Texture index
          glNormal3f (Temple_normals[ni][0],Temple_normals[ni][1],Temple_normals[ni][2]);
		  //glNormal3f (0,1,0);
           glTexCoord2f(Temple_textures[ti][0],Temple_textures[ti][1]);
           glVertex3f (100*Temple_vertices[vi][0],100*Temple_vertices[vi][1],100*Temple_vertices[vi][2]);
        }
       }
    glEnd ();

   glEndList();
   return lid;
};


