/******************************************************************************************/
/*   OPENGLWNDENGINE CLASS VER 1.0                                                        */
/*                                                                                        */
/*   This class provides the functionality required for using OpenGL Features             */
/*   ------------------------------------------------------------------------             */
/*   Ioannis Panagopoulos (c) October 2004                                                */
/*   Ver 1.0                                                                              */
/*   OpenGLWndEngine.h                                                                    */
/*                                                                                        */
/******************************************************************************************/

#include <windows.h>

#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glut/glut.h>

#include <math.h>
#include <stdio.h>

#ifndef _OPENGLTYPES_H_
#define _OPENGLTYPES_H_
#include "OPENGLTypes.h"
#endif

/****************************************/
/* Include Here Deep Exploration Data   */
/****************************************/

#include "Temple.h"

/***********************************************/
/*                                             */
/* Application Specific Defines				   */
/*                                             */
/***********************************************/

#define BULLET_LIFE 500		// The time intervals a bullet lives	
#define MAX_BULLETS 200		// The maximum number of bullets allowed on the screen
#define TERRAIN_SIZE 100	// The size of the generated terrain in tiles



/******************************************************************************************/
/******************************************************************************************/
/******************************************************************************************/
/******************************************************************************************/

class COpenGLWndEngine  
{
public:
	COpenGLWndEngine();
	virtual ~COpenGLWndEngine();
	

/******************************************************************************************/
/*                                                                                        */
/* OPENGL Initialization Functions for Windows                                            */
/*                                                                                        */
/******************************************************************************************/

	void EnableOpenGL(HWND hWnd, HDC * hDC, HGLRC * hRC);				// Enables OPENGL in a Window
	void DisableOpenGL(HWND hWnd, HDC hDC, HGLRC hRC);					// Disables OPENGL in a Window
	void ClearWindow(float r,float g,float b,float lum,float depth);	// Clears the OPENGL Window
	void WindowResized(int w,int h);									// Called whenever the window is Resized
	void Flip();														// Flips the Buffers
	void SetSceneGlobals();												// Global Sets for the screen
	void Switch3D2D(int dir);

	HDC dc;																// The captured dc (OPENGL locks)
	GLfloat w;															// The display's width	
	GLfloat h;															// The display's height

/******************************************************************************************/
/*                                                                                        */
/* OPENGL Loading and Creating Materials/Textures                                         */
/*                                                                                        */
/******************************************************************************************/

	void MyMaterial(GLenum mode,GLfloat *f,GLfloat alpha);
	void SelectMaterial(MATERIAL *materials,int i);

	IMAGE Photo;
	IMAGE Terrain;
	GLfloat heights[TERRAIN_SIZE][TERRAIN_SIZE];
	GLfloat SCALE;

	GLbyte * LoadTGA(const char *szFileName, GLint *iWidth, GLint *iHeight, GLint *iComponents, GLenum *eFormat);
	GLint TerrainList;
	void CreateTerrain();
	void LoadTexture();

	GLint fontlist;
	void CreateFonts();


/******************************************************************************************/
/*                                                                                        */
/* OPENGL Camera Manipuation                                                              */
/*                                                                                        */
/******************************************************************************************/
	ACTOR_POS Player;				// The camera Position and orientation

	void SetCamera (GLfloat xpos,GLfloat ypos,GLfloat zpos); // Set the camera pos
	void RollCamera (int dir, GLfloat step);	// Roll the camera by step (angle)		
	void MoveCamera (int dir,GLfloat step);		// Move the camera

/******************************************************************************************/
/*                                                                                        */
/* OPENGL Lights Creation                                                                */
/*                                                                                        */
/******************************************************************************************/

	Light General;
	Light Spot;
	Light SpotFixed;
	void SetLights();


/******************************************************************************************/
/*                                                                                        */
/* OPENGL Application Related Objects                                                     */
/*                                                                                        */
/******************************************************************************************/

	BOOL DrawTemple;
	vertex3 SpaceShip;
	vertex3 TemplePos;
	int mass;

	ACTOR_POS Bullets[MAX_BULLETS];
	int BulletTimer[MAX_BULLETS];
	int nbullets;

	GLfloat world_size;
	GLint hlist,hlist1;

	void CreateBullet();
	void UpdateBullets();
	void DrawBullets();

	void DrawObjects ();
	void DrawWalls (GLfloat size);
	void DrawTerrain();

	void DrawCube();
	void DrawSphere();
	void DrawEggHead();

	GLint Gen3DObjectList1();



};

static MATERIAL Appmaterials [6] = {
	{{1,1,0},		{1,1,0},	{1,1,0},		{0,0,0}, 1,2,-1}, //BULLET
	{{1,0.2,0.2},	{1,0,0},	{1,0.2,0.2},	{0,0,0},  1,2,-1}, //CUBE
	{{1,1,0.2},	{1,1,0},	{0.2,0.2,0.2},	{0,0,0},  0.5,2,-1},  //SPHERE
	{{0,0,1},		{0,0,0},	{0,0,1},		{0,0,0},  0.4,2,-1},  //WALLS
	{{1,1,1},		{1,1,1},	{1,1,1},		{1,1,1},  1,2,-1},  //LETTERS
	{{0,0,0},	{0.9f,0.9f,0.9f},	{0.9f,0.9f,0.9f},	{0.0f,0.0f,0.0f},	1.0f,2.0f,128} //BULLET
};
